@@ -483,7 +483,23 @@ static const PreferenceDefault preference_defaults[] = {
 	  NULL,
 	  default_network_link_name, g_free,
 	},
-	
+
+	{ NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT,
+	  PREFERENCE_BOOLEAN,
+	  GINT_TO_POINTER (TRUE)
+	},
+	{ NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT_OPEN,
+	  PREFERENCE_BOOLEAN,
+	  GINT_TO_POINTER (TRUE)
+	},
+	{ NAUTILUS_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_ASK,
+	  PREFERENCE_STRING_ARRAY,
+	  "", NULL, NULL, NULL
+	},
+	{ NAUTILUS_PREFERENCES_MEDIA_AUTORUN_X_CONTENT_IGNORE,
+	  PREFERENCE_STRING_ARRAY,
+	  "", NULL, NULL, NULL
+	},	
 	{ NULL }
 };
 
