@@ -37,20 +37,21 @@
 #include <gtk/gtksizegroup.h>
 
 #include <libgnome/gnome-help.h>
-#include <libgnome/gnome-i18n.h>
+#include <glib/gi18n.h>
 
 #include <glade/glade.h>
 
 #include <eel/eel-gconf-extensions.h>
 #include <eel/eel-glib-extensions.h>
 #include <eel/eel-preferences-glade.h>
-#include <eel/eel-string-list.h>
 
 #include <libnautilus-private/nautilus-column-chooser.h>
 #include <libnautilus-private/nautilus-column-utilities.h>
 #include <libnautilus-private/nautilus-global-preferences.h>
 #include <libnautilus-private/nautilus-module.h>
 
+#include <libnautilus-private/nautilus-autorun.h>
+
 /* string enum preferences */
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_DEFAULT_VIEW_WIDGET "default_view_combobox"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET "icon_view_zoom_combobox"
@@ -67,11 +68,14 @@
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_COMPACT_LAYOUT_WIDGET "compact_layout_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_LABELS_BESIDE_ICONS_WIDGET "labels_beside_icons_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET "always_use_browser_checkbutton"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_LOCATION_ENTRY_WIDGET "always_use_location_entry_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET "trash_confirm_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET "trash_delete_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_OPEN_NEW_WINDOW_WIDGET "new_window_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET "hidden_files_checkbutton"
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET "treeview_folders_checkbutton"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT "media_automount_checkbutton"
+#define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT_OPEN "media_automount_open_checkbutton"
 
 /* int enums */
 #define NAUTILUS_FILE_MANAGEMENT_PROPERTIES_THUMBNAIL_LIMIT_WIDGET "preview_image_size_combobox"
@@ -161,17 +165,6 @@ static const char * const icon_captions_components[] = {
 	NULL
 };
 
-static GladeXML *
-nautilus_file_management_properties_dialog_create (void)
-{
-	GladeXML *xml_dialog;
-
-	xml_dialog = glade_xml_new (GLADEDIR "/nautilus-file-management-properties.glade",
-				    NULL, NULL);
-
-	return xml_dialog;
-}
-
 static void
 nautilus_file_management_properties_size_group_create (GladeXML *xml_dialog,
 						       char *prefix,
@@ -266,18 +259,25 @@ static void
 columns_changed_callback (NautilusColumnChooser *chooser,
 			  gpointer callback_data)
 {
-	GList *visible_columns;
-	GList *column_order;
-	
+	char **visible_columns;
+	char **column_order;
+
 	nautilus_column_chooser_get_settings (NAUTILUS_COLUMN_CHOOSER (chooser),
 					      &visible_columns,
 					      &column_order);
 
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS, visible_columns);
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER, column_order);
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS, visible_columns);
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER, column_order);
 
-	eel_g_list_free_deep (visible_columns);
-	eel_g_list_free_deep (column_order);
+	g_strfreev (visible_columns);
+	g_strfreev (column_order);
+}
+
+static void
+free_column_names_array (GPtrArray *column_names)
+{
+	g_ptr_array_foreach (column_names, (GFunc) g_free, NULL);
+	g_ptr_array_free (column_names, TRUE);
 }
 
 static void
@@ -285,12 +285,12 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 			             GList *columns)
 {
 	GList *l;
-	EelStringList *column_names;
+	GPtrArray *column_names;
+
+	column_names = g_ptr_array_new ();
 
-	column_names = eel_string_list_new (FALSE);
-	
 	gtk_combo_box_append_text (combo_box, _("None"));
-	eel_string_list_insert (column_names, "none");
+	g_ptr_array_add (column_names, g_strdup ("none"));
 
 	for (l = columns; l != NULL; l = l->next) {
 		NautilusColumn *column;
@@ -298,7 +298,7 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 		char *label;
 
 		column = NAUTILUS_COLUMN (l->data);
-		
+
 		g_object_get (G_OBJECT (column), 
 			      "name", &name, "label", &label, 
 			      NULL);
@@ -309,48 +309,50 @@ create_icon_caption_combo_box_items (GtkComboBox *combo_box,
 			g_free (label);
 			continue;
 		}
-		
-		gtk_combo_box_append_text (combo_box, label);
-		eel_string_list_insert (column_names, name);
 
-		g_free (name);
+		gtk_combo_box_append_text (combo_box, label);
+		g_ptr_array_add (column_names, name);
+
 		g_free (label);
 	}
 	g_object_set_data_full (G_OBJECT (combo_box), "column_names",
 			        column_names,
-			        (GDestroyNotify) eel_string_list_free);
+			        (GDestroyNotify) free_column_names_array);
 }
 
 static void
 icon_captions_changed_callback (GtkComboBox *combo_box,
 				gpointer user_data)
 {
-	GList *captions;
+	GPtrArray *captions;
 	GladeXML *xml;
 	int i;
 	
 	xml = GLADE_XML (user_data);
 
-	captions = NULL;
+	captions = g_ptr_array_new ();
 
 	for (i = 0; icon_captions_components[i] != NULL; i++) {
 		GtkWidget *combo_box;
 		int active;
-		EelStringList *column_names;
+		GPtrArray *column_names;
 		char *name;
-		
+
 		combo_box = glade_xml_get_widget
 			    (GLADE_XML (xml), icon_captions_components[i]);
 		active = gtk_combo_box_get_active (GTK_COMBO_BOX (combo_box));
 
-		column_names = g_object_get_data (G_OBJECT (combo_box), "column_names");
-		
-		name = eel_string_list_nth (column_names, active);
-		captions = g_list_prepend (captions, name);
+		column_names = g_object_get_data (G_OBJECT (combo_box),
+						  "column_names");
+
+		name = g_ptr_array_index (column_names, active);
+		g_ptr_array_add (captions, name);
 	}
-	captions = g_list_reverse (captions);
-	eel_preferences_set_string_glist (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS, captions);
-	eel_g_list_free_deep (captions);
+	g_ptr_array_add (captions, NULL);
+
+	eel_preferences_set_string_array (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS,
+					  (char **)captions->pdata);
+	g_ptr_array_free (captions, TRUE);
 }
 
 static void
@@ -360,8 +362,8 @@ update_caption_combo_box (GladeXML *xml,
 {
 	GtkWidget *combo_box;
 	int i;
-	EelStringList *column_names;
-	
+	GPtrArray *column_names;
+
 	combo_box = glade_xml_get_widget (xml, combo_box_name);
 
 	g_signal_handlers_block_by_func
@@ -372,8 +374,8 @@ update_caption_combo_box (GladeXML *xml,
 	column_names = g_object_get_data (G_OBJECT (combo_box), 
 					  "column_names");
 
-	for (i = 0; i < eel_string_list_get_length (column_names); i++) {
-		if (!strcmp (name, eel_string_list_peek_nth (column_names, i))) {
+	for (i = 0; i < column_names->len; ++i) {
+		if (!strcmp (name, g_ptr_array_index (column_names, i))) {
 			gtk_combo_box_set_active (GTK_COMBO_BOX (combo_box), i);
 			break;
 		}
@@ -388,21 +390,29 @@ update_caption_combo_box (GladeXML *xml,
 static void
 update_icon_captions_from_gconf (GladeXML *xml)
 {
-	GList *captions;
-	int i;
-	GList *l;
-	
+	char **captions;
+	int i, j;
 
-	captions = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS);
+	captions = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_ICON_VIEW_CAPTIONS);
+
+	for (i = 0, j = 0; 
+	     icon_captions_components[i] != NULL;
+	     i++) {
+		char *data;
+
+		if (captions[j]) {
+			data = captions[j];
+			++j;
+		} else {
+			data = "none";
+		}
 
-	for (l = captions, i = 0; 
-	     captions != NULL && icon_captions_components[i] != NULL;
-	      l = l->next, i++) {
 		update_caption_combo_box (xml, 
 					  icon_captions_components[i],
-					  (char *)l->data);
+					  data);
 	}
-	eel_g_list_free_deep (captions);
+
+	g_strfreev (captions);
 }
 
 static void
@@ -464,19 +474,18 @@ create_date_format_menu (GladeXML *xml_dialog)
 static void
 set_columns_from_gconf (NautilusColumnChooser *chooser)
 {
-	GList *visible_columns;
-	GList *column_order;
+	char **visible_columns;
+	char **column_order;
 	
-	visible_columns = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
-	column_order = eel_preferences_get_string_glist (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
+	visible_columns = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_VISIBLE_COLUMNS);
+	column_order = eel_preferences_get_string_array (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_COLUMN_ORDER);
 
 	nautilus_column_chooser_set_settings (NAUTILUS_COLUMN_CHOOSER (chooser), 
 					      visible_columns,
 					      column_order);
 
-
-	eel_g_list_free_deep (visible_columns);
-	eel_g_list_free_deep (column_order);
+	g_strfreev (visible_columns);
+	g_strfreev (column_order);
 }
 
 static void 
@@ -508,6 +517,26 @@ nautilus_file_management_properties_dialog_setup_list_column_page (GladeXML *xml
 	gtk_box_pack_start (GTK_BOX (box), chooser, TRUE, TRUE, 0);
 }
 
+
+static void
+nautilus_file_management_properties_dialog_setup_media_page (GladeXML *xml_dialog)
+{
+	unsigned int n;
+	const char *s[] = {"media_audio_cdda_combobox",   "x-content/audio-cdda",
+			   "media_video_dvd_combobox",    "x-content/video-dvd",
+			   "media_video_vcd_combobox",    "x-content/video-vcd",
+			   "media_video_svcd_combobox",   "x-content/video-svcd",
+			   "media_blank_combobox",        "x-content/blank-media",
+			   "media_music_player_combobox", "x-content/music-player",
+			   "media_dcf_combobox",          "x-content/image-dcf",
+			   NULL};
+
+	for (n = 0; s[n*2] != NULL; n++) {
+		nautilus_autorun_prepare_combo_box (glade_xml_get_widget (xml_dialog, s[n*2]), s[n*2 + 1],
+						    TRUE, TRUE, NULL, NULL); 
+	}
+}
+
 static  void
 nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindow *window)
 {
@@ -548,6 +577,14 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_ALWAYS_USE_BROWSER_WIDGET,
 					    NAUTILUS_PREFERENCES_ALWAYS_USE_BROWSER);
+
+	eel_preferences_glade_connect_bool (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT,
+					    NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT);
+	eel_preferences_glade_connect_bool (xml_dialog,
+					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT_OPEN,
+					    NAUTILUS_PREFERENCES_MEDIA_AUTOMOUNT_OPEN);
+
 	eel_preferences_glade_connect_bool (xml_dialog,
 					    NAUTILUS_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_WIDGET,
 					    NAUTILUS_PREFERENCES_CONFIRM_TRASH);
@@ -621,6 +658,8 @@ nautilus_file_management_properties_dialog_setup (GladeXML *xml_dialog, GtkWindo
 
 	nautilus_file_management_properties_dialog_setup_icon_caption_page (xml_dialog);
 	nautilus_file_management_properties_dialog_setup_list_column_page (xml_dialog);
+
+	nautilus_file_management_properties_dialog_setup_media_page (xml_dialog);
 	
 	/* UI callbacks */
 	dialog = glade_xml_get_widget (xml_dialog, "file_management_dialog");
@@ -657,7 +696,8 @@ nautilus_file_management_properties_dialog_show (GCallback close_callback, GtkWi
 {
 	GladeXML *xml_dialog;
 
-	xml_dialog = nautilus_file_management_properties_dialog_create ();
+	xml_dialog = glade_xml_new (GLADEDIR "/nautilus-file-management-properties.glade",
+				    NULL, NULL);
 	
 	g_signal_connect (G_OBJECT (glade_xml_get_widget (xml_dialog, "file_management_dialog")),
 			  "response", close_callback, NULL);
@@ -665,4 +705,6 @@ nautilus_file_management_properties_dialog_show (GCallback close_callback, GtkWi
 			  "delete_event", G_CALLBACK (delete_event_callback), close_callback);
 
 	nautilus_file_management_properties_dialog_setup (xml_dialog, window);
+
+	g_object_unref (xml_dialog);
 }
